"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ensure_dependency_version_1 = require("./src/utilities/ensure-dependency-version");
const logger_1 = require("./src/logger/logger");
const angular_package_builder_1 = require("./src/angular-package-builder");
const packageJson = require("./package.json");
function runAngularPackageBuilder(configOrConfigUrl, debug) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            logger_1.Logger.empty();
            logger_1.Logger.title('Angular Package Builder');
            logger_1.Logger.empty();
            const startTime = new Date().getTime();
            Promise.all(Object
                .keys(packageJson.peerDependencies)
                .map((peerDependency) => {
                return ensure_dependency_version_1.ensureDependencyVersion(peerDependency, packageJson.peerDependencies[peerDependency]);
            }));
            const angularPackageBuilder = new angular_package_builder_1.AngularPackageBuilder();
            logger_1.Logger.task('Configuration');
            yield angularPackageBuilder.configure(configOrConfigUrl, debug);
            logger_1.Logger.task('Prepare', 'line breaks, resources');
            yield angularPackageBuilder.prepare();
            logger_1.Logger.task('Compile TypeScript into JavaScript', 'ES2015, ES5');
            yield Promise.all([
                angularPackageBuilder.compile('ES2015'),
                angularPackageBuilder.compile('ES5'),
            ]);
            logger_1.Logger.task('Generate JavaScript bundles', 'ES2015, ES5, UMD');
            yield Promise.all([
                angularPackageBuilder.bundle('ES2015'),
                angularPackageBuilder.bundle('ES5'),
                angularPackageBuilder.bundle('UMD')
            ]);
            logger_1.Logger.task('Compose package');
            yield angularPackageBuilder.compose();
            const finishTime = new Date().getTime();
            const processTime = ((finishTime - startTime) / 1000).toFixed(2);
            logger_1.Logger.empty();
            logger_1.Logger.success(`Success! [${processTime} seconds]`);
            logger_1.Logger.empty();
        }
        catch (error) {
            logger_1.Logger.empty();
            logger_1.Logger.error(error.message);
            logger_1.Logger.empty();
            throw new Error(error.message);
        }
    });
}
exports.runAngularPackageBuilder = runAngularPackageBuilder;
//# sourceMappingURL=index.js.map