"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const proxyquire = require("proxyquire");
const memory_file_system_1 = require("./memory-file-system/memory-file-system");
const configure_1 = require("./tasks/configure");
const delete_folder_1 = require("./utilities/delete-folder");
class AngularPackageBuilder {
    configure(configOrConfigUrl = '.angular-package.json', debug = false) {
        return __awaiter(this, void 0, void 0, function* () {
            this.config = yield configure_1.configure(configOrConfigUrl);
            this.debug = debug;
            if (this.debug) {
                yield delete_folder_1.deleteFolder(this.config.temporary.folder);
            }
            else {
                this.memoryFileSystem = new memory_file_system_1.MemoryFileSystem();
                yield this.memoryFileSystem.fill(this.config.entry.folder);
            }
            yield delete_folder_1.deleteFolder(this.config.output.folder);
        });
    }
    prepare() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.dynamicImport('./tasks/prepare').prepare(this.config);
        });
    }
    compile(target) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.dynamicImport('./tasks/compile').compile(this.config, target);
        });
    }
    bundle(target) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.dynamicImport('./tasks/bundle').bundle(this.config, target);
        });
    }
    compose() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.dynamicImport('./tasks/compose').compose(this.config);
            if (!this.debug) {
                yield this.memoryFileSystem.persist(this.config.output.folder);
            }
        });
    }
    dynamicImport(moduleDefinition) {
        return this.debug
            ? require(moduleDefinition)
            : proxyquire(moduleDefinition, {
                fs: this.memoryFileSystem.fs
            });
    }
}
exports.AngularPackageBuilder = AngularPackageBuilder;
//# sourceMappingURL=angular-package-builder.js.map