"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const parsePackageJsonName = require("parse-packagejson-name");
const logger_1 = require("../logger/logger");
function getTypescriptConfig(target, destinationPath, files, config) {
    return {
        compilerOptions: Object.assign({
            declaration: true,
            emitDecoratorMetadata: true,
            experimentalDecorators: true,
            lib: [
                'es5',
                'es6',
                'es2015',
                'es7',
                'es2016',
                'es2017',
                'esnext',
                'dom',
                'dom.iterable',
                'scripthost'
            ],
            module: 'ES2015',
            moduleResolution: 'node',
            newLine: 'LF',
            outDir: destinationPath,
            pretty: true,
            rootDir: config.temporary.prepared,
            sourceMap: true,
            sourceRoot: config.temporary.prepared,
            target
        }, validateTypescriptCompilerOptions(config.typescriptCompilerOptions)),
        files,
        angularCompilerOptions: Object.assign({
            annotateForClosureCompiler: true,
            flatModuleId: config.packageName,
            flatModuleOutFile: `${parsePackageJsonName(config.packageName).fullName}.js`,
            preserveWhitespaces: false,
            skipTemplateCodegen: true,
            strictMetadataEmit: true
        }, validateAngularCompilerOptions(config.angularCompilerOptions))
    };
}
exports.getTypescriptConfig = getTypescriptConfig;
function validateTypescriptCompilerOptions(typescriptCompilerOptions) {
    const typescriptCompilerOptionsBlacklist = [
        'declaration',
        'emitDecoratorMetadata',
        'experimentalDecorators',
        'module',
        'moduleResolution',
        'newLine',
        'outDir',
        'rootDir',
        'sourceRoot',
        'target'
    ];
    const { newCompilerOptions, removedCompilerOptions } = validateOptions(typescriptCompilerOptions, typescriptCompilerOptionsBlacklist);
    if (removedCompilerOptions.length > 0) {
        logger_1.Logger.warn(`The TypeScript compiler option ${removedCompilerOptions.join(', ')} cannot be changed.`);
    }
    return newCompilerOptions;
}
function validateAngularCompilerOptions(angularCompilerOptions) {
    const angularCompilerOptionsBlacklist = [
        'flatModuleId',
        'flatModuleOutFile',
        'skipTemplateCodegen'
    ];
    const { newCompilerOptions, removedCompilerOptions } = validateOptions(angularCompilerOptions, angularCompilerOptionsBlacklist);
    if (removedCompilerOptions.length > 0) {
        logger_1.Logger.warn(`The TypeScript compiler option ${removedCompilerOptions.join(', ')} cannot be changed.`);
    }
    return newCompilerOptions;
}
function validateOptions(compilerOptions, compilerOptionsBlacklist) {
    const removedCompilerOptions = [];
    const newCompilerOptions = Object
        .keys(compilerOptions)
        .filter((compilerOption) => {
        const isValid = compilerOptionsBlacklist.indexOf(compilerOption) === -1;
        if (!isValid) {
            removedCompilerOptions.push(compilerOption);
        }
        return isValid;
    })
        .reduce((newCompilerOptions, compilerOption) => {
        newCompilerOptions[compilerOption] = compilerOptions[compilerOption];
        return newCompilerOptions;
    }, {});
    return {
        newCompilerOptions,
        removedCompilerOptions
    };
}
//# sourceMappingURL=typescript.config.js.map