"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path_1 = require("path");
const memfs_1 = require("memfs");
const unixify = require("unixify");
const get_files_1 = require("../utilities/get-files");
const read_file_1 = require("../utilities/read-file");
const write_file_1 = require("../utilities/write-file");
class MemoryFileSystem {
    constructor() {
        this.volume = this.createVolume();
        this.fs = this.createFs(this.volume);
    }
    fill(folderPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const fileGlobs = [
                path_1.posix.join(folderPath, '**', '*.ts'),
                `!${path_1.posix.join(folderPath, '**', '*.spec.ts')}`,
                path_1.posix.join(folderPath, '**', '*.js'),
                path_1.posix.join(folderPath, '**', '*.json'),
                path_1.posix.join(folderPath, '**', '*.html'),
                path_1.posix.join(folderPath, '**', '*.sass'),
                path_1.posix.join(folderPath, '**', '*.scss'),
                path_1.posix.join(folderPath, '**', '*.css')
            ];
            const filePaths = yield get_files_1.getFiles(fileGlobs);
            const fileContents = yield Promise.all(filePaths.map((file) => {
                return read_file_1.readFile(file);
            }));
            filePaths.forEach((file, index) => {
                this.volume.mkdirpSync(path_1.posix.dirname(file));
                this.volume.writeFileSync(file, fileContents[index], 'utf-8');
            });
        });
    }
    persist(folderPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const volumeContent = this.volume.toJSON();
            const normalizedFolderPath = `${unixify(folderPath)}${path_1.posix.sep}`;
            const files = Object
                .keys(volumeContent)
                .filter((filePath) => {
                return filePath.startsWith(normalizedFolderPath);
            });
            yield Promise.all(files.map((filePath) => __awaiter(this, void 0, void 0, function* () {
                yield write_file_1.writeFile(path_1.posix.join(filePath), volumeContent[filePath]);
            })));
        });
    }
    createVolume() {
        return memfs_1.Volume.fromJSON({}, '/');
    }
    createFs(volume) {
        const volumeFs = memfs_1.createFsFromVolume(volume);
        const fsFunctionMapping = Object
            .keys(fs)
            .reduce((fsFunctionMapping, functionName) => {
            fsFunctionMapping[functionName] = volumeFs[functionName];
            return fsFunctionMapping;
        }, {
            '@global': true
        });
        const fsFunctionMappingWithExceptions = Object.assign({}, fsFunctionMapping, {
            existsSync: (path) => {
                return (path.indexOf('node_modules') === -1)
                    ? volumeFs.existsSync(path)
                    : fs.existsSync(path);
            },
            statSync: (path) => {
                return (path.indexOf('node_modules') === -1)
                    ? volumeFs.statSync(path)
                    : fs.statSync(path);
            },
            readFileSync: (path, options) => {
                return (path.indexOf('node_modules') === -1)
                    ? volumeFs.readFileSync(path, options)
                    : fs.readFileSync(path, options);
            },
            readdirSync: (path, options) => {
                return (path.indexOf('node_modules') === -1)
                    ? volumeFs.readdirSync(path, options)
                    : fs.readdirSync(path, options);
            },
        });
        return fsFunctionMappingWithExceptions;
    }
}
exports.MemoryFileSystem = MemoryFileSystem;
//# sourceMappingURL=memory-file-system.js.map