"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const rollup_1 = require("rollup");
const parsePackageJsonName = require("parse-packagejson-name");
const unixify = require("unixify");
const rollup_config_1 = require("../config/rollup.config");
const write_file_1 = require("../utilities/write-file");
function bundle(config, target) {
    return __awaiter(this, void 0, void 0, function* () {
        const sourcePath = target === 'ES2015' ? config.temporary.buildES2015 : config.temporary.buildES5;
        const rollupInputOptions = yield rollup_config_1.getRollupInputConfig(sourcePath, target, config);
        const rollupOutputOptions = rollup_config_1.getRollupOutputConfig(target === 'UMD' ? 'umd' : 'es', config);
        const bundle = yield rollup_1.rollup(rollupInputOptions);
        const { code, map } = yield bundle.generate(rollupOutputOptions);
        const normalizedSourcePath = unixify(sourcePath);
        map.sources = map.sources.map((sourcemapSourcePath) => {
            return path_1.posix.relative(normalizedSourcePath, unixify(sourcemapSourcePath));
        });
        const fileName = `${parsePackageJsonName(config.packageName).fullName}${target === 'UMD' ? '.umd' : ''}`;
        yield Promise.all([
            write_file_1.writeFile(path_1.posix.join(config.temporary[`bundle${target}`], `${fileName}.js`), code),
            write_file_1.writeFile(path_1.posix.join(config.temporary[`bundle${target}`], `${fileName}.js.map`), map)
        ]);
    });
}
exports.bundle = bundle;
//# sourceMappingURL=bundle.js.map