"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const main_1 = require("@angular/compiler-cli/src/main");
const get_files_1 = require("../utilities/get-files");
const typescript_config_1 = require("../config/typescript.config");
const write_file_1 = require("../utilities/write-file");
function compile(config, target) {
    return __awaiter(this, void 0, void 0, function* () {
        const compilationEntries = yield getCompilationEntries(config);
        const tsconfig = typescript_config_1.getTypescriptConfig(target, config.temporary[`build${target}`], compilationEntries, config);
        const tsconfigPath = path_1.posix.join(config.temporary.folder, `tsconfig.${target.toLowerCase()}.json`);
        yield write_file_1.writeFile(tsconfigPath, tsconfig);
        main_1.main(['-p', tsconfigPath], (errorLog) => {
            throw new Error([
                `An error occured while trying to compile the TypeScript sources using the Angular Compiler.`,
                ...improveAngularCompilerCliErrors(config, errorLog)
            ].join('\n'));
        });
    });
}
exports.compile = compile;
function getCompilationEntries(config) {
    return __awaiter(this, void 0, void 0, function* () {
        const dtsFilePaths = yield get_files_1.getFiles([path_1.posix.join(config.temporary.prepared, '**', '*.d.ts')]);
        return [
            path_1.posix.join(config.temporary.prepared, config.entry.file),
            ...dtsFilePaths
        ];
    });
}
function improveAngularCompilerCliErrors(config, errorLog) {
    return errorLog
        .split('\n')
        .filter((error) => {
        return error !== '';
    })
        .map((error) => {
        return error.replace(config.temporary.prepared.split(path_1.posix.sep).pop(), config.entry.folder.split(path_1.posix.sep).pop());
    })
        .map((error) => {
        return errorLog.indexOf('warning TS0') !== -1 ? `[tsickle] ${error}` : `[TypeScript] ${error}`;
    });
}
//# sourceMappingURL=compile.js.map