"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const jsonschema_1 = require("jsonschema");
const get_dependency_map_1 = require("../utilities/get-dependency-map");
const read_file_1 = require("../utilities/read-file");
const angularPackageSchema = require("../../angular-package.schema.json");
function configure(configOrConfigUrl) {
    return __awaiter(this, void 0, void 0, function* () {
        const cwd = process.cwd().replace(/\\/g, '/');
        const config = {
            cwd,
            entry: {},
            output: {},
            temporary: {
                folder: path_1.posix.join(cwd, 'dist-angular-package-builder'),
                prepared: path_1.posix.join(cwd, 'dist-angular-package-builder', 'library-prepared'),
                buildES5: path_1.posix.join(cwd, 'dist-angular-package-builder', 'library-build-es5'),
                buildES2015: path_1.posix.join(cwd, 'dist-angular-package-builder', 'library-build-es2015'),
                bundleES2015: path_1.posix.join(cwd, 'dist-angular-package-builder', 'library-bundle-es2015'),
                bundleES5: path_1.posix.join(cwd, 'dist-angular-package-builder', 'library-bundle-es5'),
                bundleUMD: path_1.posix.join(cwd, 'dist-angular-package-builder', 'library-bundle-umd')
            },
            packageName: '',
            dependencies: {},
            typescriptCompilerOptions: {},
            angularCompilerOptions: {}
        };
        const packageJsonPath = path_1.posix.join(cwd, 'package.json');
        let packageJson;
        try {
            packageJson = yield read_file_1.readFile(packageJsonPath);
        }
        catch (error) {
            throw new Error([
                `The "package.json" file at "${packageJsonPath}" does not exist, or cannot be read.`,
                `Details: ${error.message}`
            ].join('\n'));
        }
        if (!packageJson.hasOwnProperty('name')) {
            throw new Error('The "package.json" file has no "name" property.');
        }
        config.packageName = packageJson.name;
        const packageDependencies = [
            ...Object.keys(packageJson.dependencies || {}),
            ...Object.keys(packageJson.devDependencies || {}),
            ...Object.keys(packageJson.optionalDependencies || {}),
            ...Object.keys(packageJson.peerDependencies || {})
        ];
        config.dependencies = get_dependency_map_1.getDependencyMap(packageDependencies);
        let projectConfig;
        try {
            projectConfig = typeof configOrConfigUrl === 'string'
                ? yield read_file_1.readFile(path_1.posix.join(cwd, configOrConfigUrl))
                : configOrConfigUrl;
        }
        catch (error) {
            throw new Error(`The Angular Package config file at "${path_1.posix.join(cwd, configOrConfigUrl)}" does not exist.`);
        }
        const validatorResult = jsonschema_1.validate(projectConfig, angularPackageSchema);
        if (!validatorResult.valid) {
            const errorMessages = validatorResult.errors.map((error) => {
                return `${error.property.replace('instance.', '').replace('instance', '')} ${error.message}`;
            });
            throw new Error([
                'The given Angular Package Builder configuration is invalid. Make sure it follows the JSON schema.',
                errorMessages.join('\n')
            ].join('\n'));
        }
        const entryFilePath = path_1.posix.join(cwd, projectConfig.entryFile);
        try {
            yield read_file_1.readFile(entryFilePath);
        }
        catch (error) {
            throw new Error([
                `The entry file at "${entryFilePath}" does not exist, or cannot be read.`,
                `Details: ${error.message}`
            ].join('\n'));
        }
        config.entry.folder = path_1.posix.dirname(entryFilePath);
        config.entry.file = path_1.posix.basename(entryFilePath);
        if (path_1.posix.extname(config.entry.file).substring(1).toLowerCase() !== 'ts') {
            throw new Error(`The entry file at "${entryFilePath}" is not a TypeScript file.`);
        }
        config.output.folder = path_1.posix.join(cwd, projectConfig.outDir);
        config.dependencies = Object.assign({}, config.dependencies, (projectConfig.dependencies || {}));
        config.typescriptCompilerOptions = projectConfig.typescriptCompilerOptions || {};
        config.angularCompilerOptions = projectConfig.angularCompilerOptions || {};
        return config;
    });
}
exports.configure = configure;
//# sourceMappingURL=configure.js.map