"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const angular_resource_analyzer_1 = require("../resources/angular-resource-analyzer");
const compile_sass_1 = require("../resources/compile-sass");
const get_files_1 = require("../utilities/get-files");
const minify_css_1 = require("../resources/minify-css");
const minify_html_1 = require("../resources/minify-html");
const read_file_1 = require("../utilities/read-file");
const write_file_1 = require("../utilities/write-file");
function prepare(config) {
    return __awaiter(this, void 0, void 0, function* () {
        const sourceFilePatterns = [
            path_1.posix.join('**', '*.ts'),
            `!${path_1.posix.join('**', '*.spec.ts')}`
        ];
        const sourceFilePaths = yield get_files_1.getFiles(sourceFilePatterns, config.entry.folder);
        yield Promise.all(sourceFilePaths.map((sourceFilePath) => __awaiter(this, void 0, void 0, function* () {
            const absoluteSourceFilePath = path_1.posix.join(config.entry.folder, sourceFilePath);
            const absoluteOutputFilePath = path_1.posix.join(config.temporary.prepared, sourceFilePath);
            let sourceFileContent = yield read_file_1.readFile(absoluteSourceFilePath);
            sourceFileContent = transformLineEndings(sourceFileContent);
            sourceFileContent = yield transformResources(sourceFileContent, absoluteSourceFilePath);
            yield write_file_1.writeFile(absoluteOutputFilePath, sourceFileContent);
        })));
    });
}
exports.prepare = prepare;
function transformLineEndings(fileContent) {
    return fileContent.replace(/\r\n|\r|\n/g, '\n');
}
function transformResources(fileContent, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const angularResourceAnalyzer = new angular_resource_analyzer_1.AngularResourceAnalyzer(filePath, fileContent);
        let externalResources = angularResourceAnalyzer.findExternalResources();
        externalResources = yield Promise.all(externalResources.map((externalResource) => __awaiter(this, void 0, void 0, function* () {
            externalResource.urls = yield Promise.all(externalResource.urls.map((url) => __awaiter(this, void 0, void 0, function* () {
                url.content = yield getPreparedExternalResource(url.url, filePath);
                return url;
            })));
            return externalResource;
        })));
        return angularResourceAnalyzer.inlineExternalResources(externalResources);
    });
}
function getPreparedExternalResource(resourceUrl, filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const resourcePath = path_1.posix.join(path_1.posix.dirname(filePath), resourceUrl);
        let resource;
        try {
            resource = yield read_file_1.readFile(resourcePath);
        }
        catch (error) {
            throw new Error([
                `The external resource at "${resourcePath}" does not exist, or cannot be read.`,
                error.message
            ].join('\n'));
        }
        switch (path_1.posix.extname(resourceUrl).substring(1).toLowerCase()) {
            case 'html':
                resource = minify_html_1.minifyHtml(resource);
                break;
            case 'css':
                resource = minify_css_1.minifyCss(resource);
                break;
            case 'scss':
            case 'sass':
                resource = yield compile_sass_1.compileSass(resource);
                resource = minify_css_1.minifyCss(resource);
                break;
        }
        return resource;
    });
}
//# sourceMappingURL=prepare.js.map