"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const semver = require("semver");
const logger_1 = require("../logger/logger");
function ensureDependencyVersion(dependency, requiredVersion) {
    return __awaiter(this, void 0, void 0, function* () {
        let installedDependencyVersion;
        try {
            installedDependencyVersion = (yield Promise.resolve().then(function () { return require(path_1.posix.join(dependency, 'package.json')); })).version;
        }
        catch (error) {
            throw new Error([
                `The package "${dependency}" does not seem to be installed, or the installation is broken.`,
                `Make sure that "${dependency}" is listed in your dependencies, perhaps re-install them ...`,
                error.message
            ].join('\n'));
        }
        if (!semver.satisfies(installedDependencyVersion, requiredVersion)) {
            logger_1.Logger.warn([
                `Version "${installedDependencyVersion}" of "${dependency}" is not supported by the Angular Package Builder.`,
                `The dependency is expected to satisfy "${requiredVersion}". Will try to continue anyway ...`
            ].join('\n'));
        }
    });
}
exports.ensureDependencyVersion = ensureDependencyVersion;
//# sourceMappingURL=ensure-dependency-version.js.map